library(abind)
library(lattice)
# setwd("C:/Users/WFU2012/Desktop/Research/DaveEngler/RewriteSims")

last=50
nsims=10
len=10*365

hold=matrix(0,len,5)
for (i in 1:last) {
  print(i)
  for (j in 1:nsims) {
    name=paste("tempRewrite",i,"S",j,".csv",sep="")
    readin=read.csv(file=name, header=T)[,-1]
    hold=abind(hold,readin, along=3)
  }
}
tempsimstmp=hold[,,-1]

zhold=matrix(0,len,5)
for (i in 1:last) {
  print(i)
  for (j in 1:nsims) {
    name=paste("ztempRewrite",i,"S",j,".csv",sep="")
    readin=read.csv(file=name, header=T)[,-1]
    zhold=abind(zhold,readin, along=3)
  }
}
predsimstmp=zhold[,,-1] ## These are the Zs only

### Need to make only 365 rows long
tempsims = matrix(0,365,5)
for (i in 1:10) {
  print(i)
  st=(i-1)*365 + 1
  end=st+364
  for (z in 1:(last*nsims)) {
    tempsims=abind(tempsims,tempsimstmp[(st:end),,z],along=3)
  }
}
tempsims=tempsims[,,-1]

predsims = matrix(0,365,5)
for (i in 1:10) {
  print(i)
  st=(i-1)*365 + 1
  end=st+364
  for (z in 1:(last*nsims)) {
    predsims=abind(predsims,predsimstmp[(st:end),,z],along=3)
  }
}
predsims=predsims[,,-1]

## It takes awhile to process the annual simulations, 
## so the workspace is saved right here.
save.image(file="AnalyzeThirdRewriteDataLoaded.RData")

## For what follows, predsims and tempsims preserves spatial
## dependence.  predsims2 and tempsims2 do not.

obs=read.csv(file="obstemp.csv", header=T)[1:len,-1]
obs[2320,5]=obs[2319,5]
obs[2321,5]=obs[2319,5]

obsmatrix=matrix(0,365,5)
for (i in 1:10) {
  print(i)
  st=(i-1)*365 + 1
  end=st+364
  obsmatrix=abind(obsmatrix,obs[(st:end),,],along=3)
}
obsmatrix=obsmatrix[,,-1]

ct1=sample(seq(1:(nsims*last*10)))
ct2=sample(seq(1:(nsims*last*10)))
ct3=sample(seq(1:(nsims*last*10)))
ct4=sample(seq(1:(nsims*last*10)))

predsims2=aperm(abind(predsims[,1,ct1], predsims[,2,ct2], predsims[,3,ct3], predsims[,4,ct4],predsims[,5,], along=3),c(1,3,2))
tempsims2=aperm(abind(tempsims[,1,ct1], tempsims[,2,ct2], tempsims[,3,ct3], tempsims[,4,ct4],tempsims[,5,], along=3),c(1,3,2))

#################
## Predictive plots:
## Total DDs for NYC and BOS

outspatial=array()
outnonspatial=array()

cddsims=apply(((tempsims[,2:3,]-65)*(tempsims[,2:3,]>65)),c(1,3),sum)

## tempsims2 and cddsims2 ignore spatial dependence
cddsims2=apply(((tempsims2[,2:3,]-65)*(tempsims2[,2:3,]>65)),c(1,3),sum)

lowers1=array()
uppers1=array()
lowerns1=array()
upperns1=array()

for (i in 1:365) {
  lowers1[i]=quantile(cddsims[i,],0.01)
  uppers1[i]=quantile(cddsims[i,],0.99)
  lowerns1[i]=quantile(cddsims2[i,],0.01)
  upperns1[i]=quantile(cddsims2[i,],0.99)
}

par(mfrow=c(2,1))
par(oma=c(2,2,3,2),mar=c(1,4,1,1))
plot(uppers1,type="l",lty=2, lwd=1.5,xlab="",xaxt='n',ylim=c(0,60), ylab="Daily CDDs",main="Pointwise 98% Intervals, Boston and NYC Totals")
legend("topright",lty=c(2,1),legend=c("spatial","non-spatial"))
order=seq(1,365,by=1)
at1=c(1,27,57,87,117,147,177,207,237,267,297,327,357)
axis(1,at=at1,labels=order[at1])
lines(lowers1,type="l",lty=2, lwd=1.5)
lines(upperns1,type="l",lty=1, lwd=1.5)
lines(lowerns1,type="l",lty=1, lwd=1.5)

## Can I add points here
cddobs=apply(((obsmatrix[,2:3,]-65)*(obsmatrix[,2:3,]>65)),c(1,3),sum)

for (t in 1:365) {
  for (i in 1:10){
    points(t,cddobs[t,i], pch=16,,cex=0.2, col="gray")
  }
}

sumspat=0
sumnonspat=0
s1=150
e1=250

for (t in s1:e1) {
  sumspat=sumspat + sum(cddobs[t,]<lowers1[t] | cddobs[t,]>uppers1[t] )
  sumnonspat = sumnonspat + sum(cddobs[t,]<lowerns1[t] | cddobs[t,]>upperns1[t] )
}
outspatial[1]=sumspat/((e1 - s1 + 1)*10)
outnonspatial[1]=sumnonspat/((e1 - s1 + 1)*10)

## Total HDDs of the 2 cities

hddsims=apply(((65-tempsims[,2:3,])*(tempsims[,2:3,]<65)),c(1,3),sum)
ct1=sample(seq(1:(nsims*last*10)))
ct2=sample(seq(1:(nsims*last*10)))
ct3=sample(seq(1:(nsims*last*10)))
ct4=sample(seq(1:(nsims*last*10)))

## tempsims2 and hddsims2 ignore spatial dependence
tempsims2=aperm(abind(tempsims[,1,ct1], tempsims[,2,ct2], tempsims[,3,ct3], tempsims[,4,ct4],tempsims[,5,], along=3),c(1,3,2))
hddsims2=apply(((65-tempsims2[,2:3,])*(tempsims2[,2:3,]<65)),c(1,3),sum)

hlowers1=array()
huppers1=array()
hlowerns1=array()
hupperns1=array()

for (i in 1:365) {
  hlowers1[i]=quantile(hddsims[i,],0.01)
  huppers1[i]=quantile(hddsims[i,],0.99)
  hlowerns1[i]=quantile(hddsims2[i,],0.01)
  hupperns1[i]=quantile(hddsims2[i,],0.99)
}

## Don't draw it yet, want it lower left

#################
## Predictive plots:
## Total DDs for LV and TU

cddsims=apply(((tempsims[,4:5,]-65)*(tempsims[,4:5,]>65)),c(1,3),sum)
ct1=sample(seq(1:(nsims*last*10)))
ct2=sample(seq(1:(nsims*last*10)))
ct3=sample(seq(1:(nsims*last*10)))
ct4=sample(seq(1:(nsims*last*10)))
tempsims2=aperm(abind(tempsims[,1,ct1], tempsims[,2,ct2], tempsims[,3,ct3], tempsims[,4,ct4],tempsims[,5,], along=3),c(1,3,2))
cddsims2=apply(((tempsims2[,4:5,]-65)*(tempsims2[,4:5,]>65)),c(1,3),sum)

lowers=array()
uppers=array()
lowerns=array()
upperns=array()

for (i in 1:365) {
  lowers[i]=quantile(cddsims[i,],0.01)
  uppers[i]=quantile(cddsims[i,],0.99)
  lowerns[i]=quantile(cddsims2[i,],0.01)
  upperns[i]=quantile(cddsims2[i,],0.99)
}

#par(mfrow=c(2,1))
#par(oma=c(2,2,3,2),mar=c(1,4,1,1))
#plot(uppers,type="l",ylim=c(0,80), lty=2, lwd=1.5,xlab="",xaxt='n',ylab="Daily CDDs",main="Las Vegas and Tucson Totals")
#order=seq(1,365,by=1)
#at1=c(1,27,57,87,117,147,177,207,237,267,297,327,357)
#axis(1,at=at1,labels=order[at1])
#lines(lowers,type="l",lty=2, lwd=1.5)
#lines(upperns,type="l",lty=1, lwd=1.5)
#lines(lowerns,type="l",lty=1, lwd=1.5)

cddobs=apply(((obsmatrix[,4:5,]-65)*(obsmatrix[,4:5,]>65)),c(1,3),sum)

#for (t in 1:365) {
#  for (i in 1:10){
#    points(t,cddobs[t,i], pch=16,,cex=0.2, col="gray")
#  }
#}

sumspat=0
sumnonspat=0
s1=125
e1=275

for (t in s1:e1) {
  sumspat=sumspat + sum(cddobs[t,]<lowers[t] | cddobs[t,]>uppers[t] )
  sumnonspat = sumnonspat + sum(cddobs[t,]<lowerns[t] | cddobs[t,]>upperns[t] )
}
outspatial[3]=sumspat/((e1 - s1 + 1)*10)
outnonspatial[3]=sumnonspat/((e1 - s1 + 1)*10)


plot(huppers1,type="l",lty=2, lwd=1.5,xlab="",xaxt='n',ylab="Daily HDDs",ylim=c(0,110) )
order=seq(1,365,by=1)
at1=c(1,27,57,87,117,147,177,207,237,267,297,327,357)
axis(1,at=at1,labels=order[at1])

lines(hlowers1,type="l",lty=2, lwd=1.5)
lines(hupperns1,type="l",lty=1, lwd=1.5)
lines(hlowerns1,type="l",lty=1, lwd=1.5)

hddobs=apply(((65-obsmatrix[,2:3,])*(obsmatrix[,2:3,]<65)),c(1,3),sum)

for (t in 1:365) {
  for (i in 1:10){
    points(t,hddobs[t,i], pch=16,,cex=0.2, col="gray")
  }
}

sumspat=0
sumnonspat=0
s1=1
e1=120
s2=300
e2=365

for (t in s1:e1) {
  sumspat=sumspat + sum(hddobs[t,]<hlowers1[t] | hddobs[t,]>huppers1[t] )
  sumnonspat = sumnonspat + sum(hddobs[t,]<hlowerns1[t] | hddobs[t,]>hupperns1[t] )
}
for (t in s2:e2) {
  sumspat=sumspat + sum(hddobs[t,]<hlowers1[t] | hddobs[t,]>huppers1[t] )
  sumnonspat = sumnonspat + sum(hddobs[t,]<hlowerns1[t] | hddobs[t,]>hupperns1[t] )
}

outspatial[2]=sumspat/((e1 - s1 + e2 - s2 + 2)*10)
outnonspatial[2]=sumnonspat/((e1 - s1 + e2 - s2 + 2)*10)

# legend("topright",lty=c(3,1),legend=c("spatial","non-spatial"))
# title("Pointwise 98% Intervals: Boston and NYC Totals",outer="TRUE")
## Total HDDs of the 2 cities

hddsims=apply(((65-tempsims[,4:5,])*(tempsims[,4:5,]<65)),c(1,3),sum)
ct1=sample(seq(1:(nsims*last*10)))
ct2=sample(seq(1:(nsims*last*10)))
ct3=sample(seq(1:(nsims*last*10)))
ct4=sample(seq(1:(nsims*last*10)))
tempsims2=aperm(abind(tempsims[,1,ct1], tempsims[,2,ct2], tempsims[,3,ct3], tempsims[,4,ct4],tempsims[,5,], along=3),c(1,3,2))
hddsims2=apply(((65-tempsims2[,4:5,])*(tempsims2[,4:5,]<65)),c(1,3),sum)

hlowers=array()
huppers=array()
hlowerns=array()
hupperns=array()

for (i in 1:365) {
  hlowers[i]=quantile(hddsims[i,],0.01)
  huppers[i]=quantile(hddsims[i,],0.99)
  hlowerns[i]=quantile(hddsims2[i,],0.01)
  hupperns[i]=quantile(hddsims2[i,],0.99)
}

#plot(huppers,type="l",lty=2, lwd=1.5,xlab="",xaxt='n',ylab="Daily HDDs",ylim=c(0,110))
#order=seq(1,365,by=1)
#at1=c(1,27,57,87,117,147,177,207,237,267,297,327,357)
#axis(1,at=at1,labels=order[at1])
#lines(hlowers,type="l",lty=2, lwd=1.5)
#lines(hupperns,type="l",lty=1, lwd=1.5)
#lines(hlowerns,type="l",lty=1, lwd=1.5)

hddobs=apply(((65-obsmatrix[,4:5,])*(obsmatrix[,4:5,]<65)),c(1,3),sum)

#for (t in 1:365) {
#  for (i in 1:10){
#    points(t,hddobs[t,i], pch=16,,cex=0.2, col="gray")
#  }
#}

sumspat=0
sumnonspat=0
s1=1
e1=75
s2=300
e2=365

for (t in s1:e1) {
  sumspat=sumspat + sum(hddobs[t,]<hlowers[t] | hddobs[t,]>huppers[t] )
  sumnonspat = sumnonspat + sum(hddobs[t,]<hlowerns[t] | hddobs[t,]>hupperns[t] )
}
for (t in s2:e2) {
  sumspat=sumspat + sum(hddobs[t,]<hlowers[t] | hddobs[t,]>huppers[t] )
  sumnonspat = sumnonspat + sum(hddobs[t,]<hlowerns[t] | hddobs[t,]>hupperns[t] )
}

outspatial[4]=sumspat/((e1 - s1 + e2 - s2 + 2)*10)
outnonspatial[4]=sumnonspat/((e1 - s1 + e2 - s2 + 2)*10)

## title("Pointwise 98% Intervals",outer="TRUE")
## Save this as pred1.eps, the combined image.


######################################
## This part aggregates CDDs and HDDs by Month, for tables.

library(ggplot2)
library(ggExtra)
library(gridExtra)

numsim=dim(tempsims)[3]
spcpay=array()
nspcpay=array()
sphpay=array()
nsphpay=array()

## First month is August, then they go in order
monthlength=c(31,28,31,30,31,30,31,31,30,31,30,31)
cumsum(monthlength)+1
range=list(1:31,32:59,60:90,91:120,121:151,152:181,182:212,213:243,244:273,274:304,305:334,335:365)
month=c("January","February","March","April","May","June","July","August","September","October","November","December")
############ CDDs
cddvar=array()
cddvar2=array()
p95=array()
p952=array()
p99=array()
p992=array()
m=array()
m2=array()

#######
## Here we compute monthly standard deviations, VaRs, and verify the equivalence of means
## for total CDDs in the 5 cities highlighted.

par(mfrow=c(3,2),oma=c(2,2,2,2),mar=c(4,4,2,2))
## With spatial dependence
for (i in 1:12) {
  cddsims=apply(((tempsims[range[[i]],,]-65)*(tempsims[range[[i]],,]>65)),c(2,3),sum)
  tcddsims=apply(cddsims, 2, sum)
  cddvar[i]=sqrt(var(tcddsims))
  p95[i]=quantile(tcddsims,0.95)
  p99[i]=quantile(tcddsims,0.99)
  m[i]=mean(tcddsims)
  
  ## w/o spatial dependence
  ct1=sample(seq(1:(nsims*last*10)))
  ct2=sample(seq(1:(nsims*last*10)))
  ct3=sample(seq(1:(nsims*last*10)))
  ct4=sample(seq(1:(nsims*last*10)))
  tempsims2=aperm(abind(tempsims[,1,ct1], tempsims[,2,ct2], tempsims[,3,ct3], tempsims[,4,ct4],tempsims[,5,], along=3),c(1,3,2))
  dim(tempsims2)
  cddsims2=apply(((tempsims2[range[[i]],,]-65)*(tempsims2[range[[i]],,]>65)),c(2,3),sum)
  tcddsims2=apply(cddsims2, 2, sum)
  cddvar2[i]=sqrt(var(tcddsims2))
  p952[i]=quantile(tcddsims2,0.95)
  p992[i]=quantile(tcddsims2,0.99)
  m2[i]=mean(tcddsims2)
  
if (i == 8) {
  df <- data.frame(x=cddsims[2,], y=cddsims[3,])
  p1 <- ggplot(df, aes(x, y)) + geom_point(size = 0.4, stroke = 0, shape = 16) + theme_classic() + xlab("Boston") + ylab("NYC") + ggtitle("Predictive August CDDs") 
  ggExtra::ggMarginal(p1, type = "histogram")
  }
if (i == 10) {
  df <- data.frame(x=cddsims[2,], y=cddsims[3,])
  p2 <- ggplot(df, aes(x, y)) + geom_point(size = 0.4, stroke = 0, shape = 16) + theme_classic() + xlab("Boston") + ylab("NYC") + ggtitle("Predictive October CDDs") 
   ggExtra::ggMarginal(p2, type = "histogram")
  }
}
grid.arrange(ggMarginal(p1, type = "histogram"), ggMarginal(p2, type = "histogram"),ncol=2)
## Save this as pred3.eps


cddvar=round(cddvar,1)
cddvar2=round(cddvar2,1)
p95=round(p95,1)
p99=round(p99,1)
p952=round(p952,1)
p992=round(p992,1)
m
m2




########################################
## HDDs
hddvar=array()
hddvar2=array()
p95h=array()
p952h=array()
p99h=array()
p992h=array()
mh=array()
m2h=array()

par(mfrow=c(3,2),oma=c(2,2,2,2),mar=c(4,4,2,2))
## With spatial dependence
for (i in 1:12) {
  hddsims=apply(((65-tempsims[range[[i]],,])*(tempsims[range[[i]],,]<65)),c(2,3),sum)
  thddsims=apply(hddsims, 2, sum)
  hddvar[i]=sqrt(var(thddsims))
  p95h[i]=quantile(thddsims,0.95)
  p99h[i]=quantile(thddsims,0.99)
  mh[i]=mean(thddsims)
  
  ## w/o spatial dependence
  ct1=sample(seq(1:(nsims*last*10)))
  ct2=sample(seq(1:(nsims*last*10)))
  ct3=sample(seq(1:(nsims*last*10)))
  ct4=sample(seq(1:(nsims*last*10)))
  tempsims2=aperm(abind(tempsims[,1,ct1], tempsims[,2,ct2], tempsims[,3,ct3], tempsims[,4,ct4],tempsims[,5,], along=3),c(1,3,2))
  dim(tempsims2)
  hddsims2=apply(((65-tempsims2[range[[i]],,])*(tempsims2[range[[i]],,]<65)),c(2,3),sum)
  thddsims2=apply(hddsims2, 2, sum)
  hddvar2[i]=sqrt(var(thddsims2))
  p952h[i]=quantile(thddsims2,0.95)
  p992h[i]=quantile(thddsims2,0.99)
  m2h[i]=mean(thddsims2)

  if (i == 1 || i==2 || i==3 || i==10 || i==11 || i==12) {
  plot(hddsims[2,],hddsims[3,], xlab="Boston",ylab="New York",main=month[i],cex=0.6,cex.main=1.3,cex.lab=1.3,cex.axis=1.3)
  }
}
title("Projected Heating Degree Days: Spatial Model", outer="TRUE")

hddvar=round(hddvar,1)
hddvar2=round(hddvar2,1)
p95h=round(p95h,1)
p99h=round(p99h,1)
p952h=round(p952h,1)
p992h=round(p992h,1)
mh
m2h

inp=cbind(p95,p99)

tabler=function(inp) {
  row=dim(inp)[1]
  col=dim(inp)[2]
  amp = rep("&",row)
  sl = rep("\\",row)
  out=array()
  for (i in 1:(col-1)) {
    out=cbind(out,inp[,i],amp)
  }
  out=out[,-1]
  out=cbind(out, inp[,col],sl)
  return(out)
}

keep=tabler(cbind(month,cddvar2, cddvar, round(100*(cddvar-cddvar2)/cddvar2,1), p952, p95, round(100*(p95-p952)/p952,1),p992, p99, round(100*(p99-p992)/p992,1)))
keep
write.csv(file="keep.csv",keep)

keeph=tabler(cbind(month,hddvar2, hddvar, round(100*(hddvar-hddvar2)/hddvar2,1),p952h, p95h, round(100*(p95h-p952h)/p952h,1), p992h, p99h,round(100*(p99h-p992h)/p992h,1)))
keeph
write.csv(file="keeph.csv",keeph)


###########################################################
## Histograms of spatial and non-spatial density estimation
##
dev.off()
par(mfrow=c(1,2))
i=9
cddsims=apply(((tempsims[range[[i]],,]-65)*(tempsims[range[[i]],,]>65)),c(2,3),sum)
tcddsims=apply(cddsims, 2, sum)
cddsims2=apply(((tempsims2[range[[i]],,]-65)*(tempsims2[range[[i]],,]>65)),c(2,3),sum)
tcddsims2=apply(cddsims2, 2, sum)

plot(density(tcddsims2), lwd=1.5, ylim=c(0,0.0035),
     main="Total September CDDs", xlab="CDDs", lty=1)
lines(density(tcddsims), lwd=1.5, lty=2)
legend("topleft",legend=c("Spatial","Non-Spatial"),lwd=c(1.5,1.5),lty=c(2,1),bty="n")
abline(v=quantile(tcddsims2,0.95), lty=1)
abline(v=quantile(tcddsims,0.95), lty=2)
abline(v=quantile(tcddsims2,0.99), lty=1)
abline(v=quantile(tcddsims,0.99), lty=2)

i=1
hddsims=apply(((65-tempsims[range[[i]],,])*(tempsims[range[[i]],,]<65)),c(2,3),sum)
thddsims=apply(hddsims, 2, sum)
hddsims2=apply(((65-tempsims2[range[[i]],,])*(tempsims2[range[[i]],,]<65)),c(2,3),sum)
thddsims2=apply(hddsims2, 2, sum)

plot(density(thddsims), lwd=1.5,
     main="Total January HDDs", xlab="HDDs",ylim=c(0,0.0035), lty=2)
lines(density(thddsims2), lty=1, lwd=1.5)
legend("topleft",legend=c("Spatial","Non-Spatial"),lwd=c(1.5,1.5),lty=c(2,1),bty="n")
abline(v=quantile(thddsims2,0.95), lty=1)
abline(v=quantile(thddsims,0.95), lty=2)
abline(v=quantile(thddsims2,0.99), lty=1)
abline(v=quantile(thddsims,0.99), lty=2)

## save as pred5.eps


##############################################
## Compare predictions to test set


preds=read.csv(file="preds.csv", header=T)[1:len,-1]

## Look at a single city:
## Spatial and non-spatial should have roughly the same MSE when compared to predicted temperatures

sqrt(apply((obs-preds)^2,2,mean))

mse=numeric()
mse2=numeric()
cors=matrix(0,5,5)
cors2=matrix(0,5,5)
zcors=matrix(0,5,5)
zcors2=matrix(0,5,5)

for (j in 1:dim(tempsims)[3]) {
  tm=sqrt(apply((predsims[,,j])^2,2,mean))
  tm2=sqrt(apply((predsims2[,,j])^2,2,mean))
  mse=rbind(mse,tm)
  mse2=rbind(mse2,tm2)
  tmp=cor(tempsims[,,j])
  tmp2=cor(tempsims2[,,j])
  ztmp=cor(predsims[,,j])
  ztmp2=cor(predsims2[,,j])
  cors=abind(cors, tmp, along=3)
  cors2=abind(cors2, tmp2, along=3)
  zcors=abind(zcors, ztmp, along=3)
  zcors2=abind(zcors2, ztmp2, along=3)
}
cors=cors[,,-1]
zcors=zcors[,,-1]
cors2=cors2[,,-1]
zcors2=zcors2[,,-1]


######### Results for MSE table
round(sqrt(apply((obs-preds)^2,2,mean)),3)
round(apply(mse,2,mean),3) ## Wow!  What a match.
apply(mse2,2,mean) ## Clearly, this still matches.


######### Results for spatial match
round(cor(obs),2) ## Test Data
A=round(apply(cors,c(1,2),mean),3)
B=round(apply(cors,c(1,2),sd),3)
A
B
A2=round(apply(cors2,c(1,2),mean),3)
B2=round(apply(cors2,c(1,2),sd),3)
A2
B2

hold2=tabler(cbind(A[,1],B[,1],A[,2],B[,2],A[,3],B[,3],A[,4],B[,4],A[,5],B[,5]))
hold2
write.csv(file="hold2.csv",hold2)

hold3=tabler(cbind(A2[,1],B2[,1],A2[,2],B2[,2],A2[,3],B2[,3],A2[,4],B2[,4],A2[,5],B2[,5]))
hold3
write.csv(file="hold3.csv",hold3)

####################################
## Compare predictions and holdout obs data

b05=matrix(0,365,5)
b25=matrix(0,365,5)
b50=matrix(0,365,5)
b75=matrix(0,365,5)
b95=matrix(0,365,5)

for (j in 1:5) {
  for (i in 1:365){
    b05[i,j]=quantile(tempsims[i,j,],0.05)
    b25[i,j]=quantile(tempsims[i,j,],0.25)
    b50[i,j]=quantile(tempsims[i,j,],0.50)
    b75[i,j]=quantile(tempsims[i,j,],0.75)
    b95[i,j]=quantile(tempsims[i,j,],0.95)
  }
}

x=seq(1,365,by=1)
x3=rep(seq(1,365,by=1),10)

dim(obs)

par(mfrow=c(1,5))
for (j in 1:5) {
  plot(x3,obs[,j], ylim=c(0,105), cex=0.5, pch=16, lty=3, main=city[use[j]],xlab="",ylab="",xaxt="n", col="gray")
  order=seq(1,365,by=1)
  at1=c(1,27,57,87,117,147,177,207,237,267,297,327,357)
  axis(1,at=at1,labels=order[at1])
  points(b05[,j], cex=1, pch=3,type="l")
  #points(b25[,j], cex=0.3, type="l", lty=2)
  #points(x,b50[,j], cex=0.3, col="red")
  #points(b75[,j], cex=0.3, type="l", lty=2)
  points(b95[,j], cex=1, pch=3, type="l")
}
## save as pred6.eps

