library(MASS)
library(abind)
library(rugarch)
library(maps)
library(date)

setwd("C:/Users/erhardrj/Desktop/NAAJCode")

## Names of the 30 .csv files downloaded from NCDC
names=c(
  "384603",
  "384607",
  "384609",
  "384613",
  "384615",
  "384618",
  "384619",
  "384621",
  "384622",
  "384625",
  "384627",
  "384636",
  "384632",
  "384635",
  "384636",
  "384640",
  "384641",
  "384642",
  "384643",
  "384680",
  "384683",
  "384684",
  "384687",
  "384689",
  "384691",
  "384693",
  "384695",
  "384704",
  "384707",
  "384711"
)

## Read in the files.
mindata=array()
maxdata=array()
dates=array()
data=list()
station=array()
for (i in 1:30) {
  hold=read.csv(file=paste(names[i],".csv", sep=""), header=T)
  data[[i]]=hold
  last=dim(hold)[1]
  sta=cbind(hold[last,1:5]);print(sta)
  station=rbind(station,sta)
  mindata[i]=min(data[[i]][,6])
  maxdata[i]=max(data[[i]][,6])
}
station=station[-1,]
city=c("Atlanta","Baltimore","Boston","Chicago","Cincinnati","Colorado Spr.","Dallas",
       "Des Moines","Detroit","Houston","Jacksonville","Kansas City","Las Vegas","Little Rock",
       "Los Angeles","Minn-St. Paul","New York","Philadelphia","Portland","Raleigh-Durham","Sacramento",
       "Salt Lake City","Tucson","Wash D.C.","Calgary","Edmonton","Montreal","Toronto","Vancouver","Winnipeg")
station=cbind(city, station)

## Make Figure 1 in the manuscript
dev.off()
use=c(1,3,17,13,23)
map(region=c("usa","canada"), xlim=c(-135,-60), ylim=c(25,58), lwd=2)
lab=c("ATL","BO","NY","LV","TU")
for (i in 1:5) {
  text(as.numeric(station[use[i],6]), as.numeric(station[use[i],5]), lab[i], pos=3, cex=1)
}
points(station[,6],station[,5], pch=16, cex=1.4)
title("Locations of the 30 Cities", cex=1.4)

## Not every city has a complete data record over the period from January 1, 1945 through 
## August 20, 2014.  Atlanta does have a complete record, so it's convenient to use Atlanta 
## to define all dates.
dates=unique(data[[1]][,6]) ## This is Atlanta, column 6 contains dates

## Here we make large matrices for tmax and tmin, and these will fill in NA for day.city combinations
## with missing data.  Missing values will be NA so they can be ignored when estimating covariance matrices.
## Warning, this is pretty time consuming.  Maybe 30 minutes.

tmax=array()
for (j in 1:29) {
  print(j)
  hold=rep(NA,length(dates))
  for (i in 1:length(dates)) {
    if(  (data[[j]][,9][data[[j]][,6]==dates[i]]>-1000) &&  (length(data[[j]][,9][data[[j]][,6]==dates[i]])>0)) {hold[i]=data[[j]][,9][data[[j]][,6]==dates[i]]} else {}
  }
  tmax=cbind(tmax,hold)
}
tmax=cbind(dates,tmax[,-1])

tmin=array()
for (j in 1:29) {
  print(j)
  hold=rep(NA,length(dates))
  for (i in 1:length(dates)) {
    if(  (data[[j]][,10][data[[j]][,6]==dates[i]]>-1000) &&  (length(data[[j]][,10][data[[j]][,6]==dates[i]])>0)) {hold[i]=data[[j]][,10][data[[j]][,6]==dates[i]]} else {}
  }
  tmin=cbind(tmin,hold)
}
tmin=cbind(dates,tmin[,-1])

## Remove all instances of Feb 29 from data record.  This is common practice when using
## sin & cosine explanatory variables to capture seasonality in the mean and variance.
substrRight <- function(x, n){
  substr(x, nchar(x)-n+1, nchar(x))
}
keep=1-1*(substrRight(tmin[,1],4)=="0229")
tminold=tmin
tmaxold=tmax
tmin=tmin[(keep==1),]
tmax=tmax[(keep==1),]
dates=dates[(keep==1)]

## Long story short, ddf contains degree-days for each day, and the units are Fahrenheit.
dd=(tmax[,2:30]+tmin[,2:30])/2
cor(dd, use="pairwise.complete.obs")
ddf=(dd/10)*9/5 + 32
ddf=ddf[1:25185,]

end=dim(ddf)[1] - 10*365## Leave out period is ten years
endp=dim(ddf)[1]

time=seq(1,end,by=1)
timep=seq(end+1,endp,by=1)
df=rep(1:365,100)[1:end]
dp=rep(1:365,10)
save.image(file="DataGARCHPreProcessing.RData")

##########################################################################
### The workspace called DataGARCHPreProcessing.RData is saved right here.
##########################################################################

## This code identifies starting and ending values for each city, and imputes interior NA 
## values.  The result is an unbroken chain of values from start to endpoint.
## Imputed values simply repeat the previous day's temperature.  Not the most elegant, but
## simple and Table 1 in the manuscript shows very few interior missing values anyway.
out=array()
for (m in 1:29) {
  hold=1*(is.na(ddf[,m]))
  out=cbind(out,hold)
}
out=out[,-1]

start=c(1,1,1,1,4501, 1186, 1096, 182, 1096, 305,
        1,1,1344,1,1,1,5142,1096,1,1,
        670,1096,1096,1308,1,5795,1,1,1)

endpointdata=c(rep(endp,24),24647,24556,endp,24983,24982)
endpoint = rep(end,29)

## The purpose of ddf2 is to impute interior missing values of the data.
## It simply repeats yesterday's measurement.  Might need to revisit this later.

far=as.matrix(dist(cbind(as.numeric(station[(1:29),5]),as.numeric(station[(1:29),6]))))
far2=far+t(far)

close=numeric()
for (m in 1:29) {
  close=cbind(close,order(far2[,m])) 
}

ctmis=matrix(0,endp,29)
ddf2=ddf
for (m in 1:29) {
  print(m)
  for (i in start[m]:endpointdata[m]) {
    if(is.na(ddf[i,m])) {
      fixit=0
      for (z in 2:29) {
        if (fixit==0 & is.na(ddf[i,close[z,m]])==FALSE) {ddf2[i,m] = as.numeric(ddf[i,close[z,m]]); fixit=1} else {}
      }
    ctmis[i,m]=1
    } else {}
  }
} 

sum(ctmis)

plot(ctmis[,23])
100*round(apply(ctmis,2,sum)/(endpoint - start),4)

##
write.csv(file="ddf2",ddf2)
head(ddf2)

## These are the external covariates for the mean and variance in the modeling.  We won't need them all.
#xmreg=cbind(time, sin(2*pi*df/365), cos(2*pi*df/365), sin(4*pi*df/365), cos(4*pi*df/365), sin(6*pi*df/365), cos(6*pi*df/365), sin(8*pi*df/365), cos(8*pi*df/365))
#xvreg=cbind(sin(2*pi*df/365), cos(2*pi*df/365),sin(4*pi*df/365), cos(4*pi*df/365),sin(6*pi*df/365), cos(6*pi*df/365), sin(8*pi*df/365), cos(8*pi*df/365))
#xmregp=cbind(timep, sin(2*pi*dp/365), cos(2*pi*dp/365), sin(4*pi*dp/365), cos(4*pi*dp/365), sin(6*pi*dp/365), cos(6*pi*dp/365), sin(8*pi*dp/365), cos(8*pi*dp/365))
#xvregp=cbind(sin(2*pi*dp/365), cos(2*pi*dp/365),sin(4*pi*dp/365), cos(4*pi*dp/365),sin(6*pi*dp/365), cos(6*pi*dp/365), sin(8*pi*dp/365), cos(8*pi*dp/365))

##################################################
## OK, here's a big transition.  Now it's time to fit the full ARMA-GARCH model
## with seasonally varying mean and variance functions for each of the 29 cities.
## First step is to make a decision about the number of sin/cos pairs to use for modeling
## the mean temperature.  Here we show that 3 (which follows Campbell and Diebold in JASA)
## is a sensible thing to do based on AIC and BIC.
# pairsAIC=array()
#pairsBIC=array()
#for (m in 1:29) {
#  Z=ddf2[start[m]:endpoint[m],m]
#  lmtry=lm(Z~xmreg[start[m]:endpoint[m],1:3])
#  p1=AIC(lmtry)
#  lmtry=lm(Z~xmreg[start[m]:endpoint[m],1:5])
#  p2=AIC(lmtry)
#  lmtry=lm(Z~xmreg[start[m]:endpoint[m],1:7])
#  p3=AIC(lmtry)
#  lmtry=lm(Z~xmreg[start[m]:endpoint[m],1:9])
#  p4=AIC(lmtry)
#  if (p1==min(p1,p2,p3,p4)) {pairsAIC[m]=1} else {}
#  if (p2==min(p1,p2,p3,p4)) {pairsAIC[m]=2} else {}
#  if (p3==min(p1,p2,p3,p4)) {pairsAIC[m]=3} else {}
#  if (p4==min(p1,p2,p3,p4)) {pairsAIC[m]=4} else {}
#}
#pairsAIC

#for (m in 1:29) {
#  Z=ddf2[start[m]:endpoint[m],m]
#  lmtry=lm(Z~xmreg[start[m]:endpoint[m],1:3])
#  p1=BIC(lmtry)
#  lmtry=lm(Z~xmreg[start[m]:endpoint[m],1:5])
#  p2=BIC(lmtry)
#  lmtry=lm(Z~xmreg[start[m]:endpoint[m],1:7])
#  p3=BIC(lmtry)
#  lmtry=lm(Z~xmreg[start[m]:endpoint[m],1:9])
#  p4=BIC(lmtry)
#  if (p1==min(p1,p2,p3,p4)) {pairsBIC[m]=1} else {}
#  if (p2==min(p1,p2,p3,p4)) {pairsBIC[m]=2} else {}
#  if (p3==min(p1,p2,p3,p4)) {pairsBIC[m]=3} else {}
#  if (p4==min(p1,p2,p3,p4)) {pairsBIC[m]=4} else {}
#}
#pairsBIC
#hist(pairsBIC)

## To recap, the argument is the BIC is more parsimonius, and the majority of cities (16) have P=3.
## Also precedent in the JASA and Benth papers.  So let's redefine external covariates:
xmreg=cbind(time, sin(2*pi*df/365), cos(2*pi*df/365), sin(4*pi*df/365), cos(4*pi*df/365), sin(6*pi*df/365), cos(6*pi*df/365))
xvreg=cbind(sin(2*pi*df/365), cos(2*pi*df/365),sin(4*pi*df/365), cos(4*pi*df/365),sin(6*pi*df/365), cos(6*pi*df/365))
xmregp=cbind(timep, sin(2*pi*dp/365), cos(2*pi*dp/365), sin(4*pi*dp/365), cos(4*pi*dp/365), sin(6*pi*dp/365), cos(6*pi*dp/365))
xvregp=cbind(sin(2*pi*dp/365), cos(2*pi*dp/365),sin(4*pi*dp/365), cos(4*pi*dp/365),sin(6*pi*dp/365), cos(6*pi*dp/365))

## We will ultimately need the first and final 3 daily residuals when we start n-step ahead prediction, 
## so we just save them here.
first3=numeric()
last3=numeric()
for(m in 1:29) {
  Z=ddf2[start[m]:endpoint[m],m]
  lmtry=lm(Z~xmreg[start[m]:endpoint[m],])
  err=lmtry$resid
  ln=length(err)
  last3=cbind(last3,err[(ln-2):ln])
  first3=cbind(first3,err[1:3])
}
first=first3
last=last3

## Here's a big section of code that ultimately helps us settle a fundamental question: should
## the modeling be one-step or two-step?  What I mean is, should we first model the seasonally-varying
## mean (parameter is called \theta in this script) and then in a second step fit the ARMA-GARCH process
## (whose parameter is called \eta in this script)?  Or, should it be a one-step maximization
## procedure that fits the joint (\theta, \eta) parameter in one go?
##
## This section of code will demonstrate that 2-step fitting is preferrable.
##
# m=1
# Z=ddf2[start[m]:endpoint[m],m]

## 2-step
#lmtry=lm(Z~xmreg[start[m]:endpoint[m],])
#err=lmtry$resid
#sim.spec <- ugarchspec(mean.model=list(armaOrder=c(25,0), include.mean=FALSE),
#                       variance.model=list(model="eGARCH", garchOrder=c(1,1),
#                                           external.regressors=xvreg[start[m]:endpoint[m],], variance.targeting=FALSE),
#                       distribution.model = "norm")
#print(m)
#gfit2 <- ugarchfit(sim.spec, err, solver="hybrid")

## 1-step

#sim.spec <- ugarchspec(mean.model=list(armaOrder=c(25,0), external.regressors=xmreg[start[m]:endpoint[m],]),
#                       variance.model=list(model="eGARCH", garchOrder=c(1,1),
#                                           external.regressors=xvreg[start[m]:endpoint[m],], variance.targeting=FALSE),
#                       distribution.model = "norm")

#gfit1 <- ugarchfit(sim.spec, Z, solver="hybrid")

## Explore the quality of the fit
#nsim=2500
#sims2 = ugarchsim(gfit2, n.sim = nsim, vexsimdata=list(xvreg[1:nsim,]))
#sims1 = ugarchsim(gfit1, n.sim = nsim, vexsimdata=list(xvreg[1:nsim,]),mexsimdata=list(xmreg[1:nsim,]))

#plot(fitted(sims2), type="l")
#plot(lmtry$resid[1:1500], type="l")

#plot(Z[1:nsim], cex=0.5)
#points(fitted(sims1), col="blue", cex=0.5)
#ms=cbind(1,xmreg)%*%lmtry$coefficients
#points(ms[1:nsim] + fitted(sims2), col="red", cex=0.5)

#plot(sigma(sims1), type="l")
#plot(sigma(sims2)[1:1500], type="l")
#hold=cbind(1,xmreg)%*%coef(gfit1)[c(1,27:33)]
#plot((fitted(sims1)-hold[1:nsim])^2, type="l")

## I conclude I should be doing 2-step fitting. 
## And this makes the parametric bootstrapping all the nicer later on.

meanparamsout=list()
meanparamssdout=list()
garchparamsout=list()
garchdata=list()
fits=list()
resids=list() 

garchparamsout2=list()
fits2=list()
resids2=list()

garchparamsout3=list()
fits3=list()
resids3=list()

## These are the residuals from the ARMA-EGARCH fit

#  for (q in 1:5) { ## Comment these two lines out when you want to run for the full 29
#  m = use[q]     ##
for (m in 1:29) {
  Z=ddf2[start[m]:endpoint[m],m]
  lmtry=lm(Z~xmreg[start[m]:endpoint[m],])
  meanparamsout[[m]]=lmtry$coef
  err=lmtry$resid
  garchdata[[m]]=err
  sim.spec1 <- ugarchspec(mean.model=list(armaOrder=c(3,0), include.mean=FALSE),
                         variance.model=list(model="eGARCH", garchOrder=c(1,1),
                                             external.regressors=xvreg[start[m]:endpoint[m],], variance.targeting=FALSE),
                         distribution.model = "norm",
                         start.pars=list(beta1=0.75, omega=.75))
  
  sim.spec2 <- ugarchspec(mean.model=list(armaOrder=c(3,0), include.mean=FALSE),
                          variance.model=list(model="eGARCH", garchOrder=c(2,1),
                                              external.regressors=xvreg[start[m]:endpoint[m],], variance.targeting=FALSE),
                          distribution.model = "norm",
                          start.pars=list(beta1=0.75, omega=.75))
 
  sim.spec3 <- ugarchspec(mean.model=list(armaOrder=c(3,0), include.mean=FALSE),
                          variance.model=list(model="eGARCH", garchOrder=c(3,1),
                                              external.regressors=xvreg[start[m]:endpoint[m],], variance.targeting=FALSE),
                          distribution.model = "norm",
                          start.pars=list(beta1=0.75, omega=.75)) 
  
  print(m)
  
  gfit <- ugarchfit(sim.spec1, err, solver="solnp")
  fits[[m]]=gfit
  resids[[m]]=as.numeric(residuals(gfit, standardize="TRUE"))
  garchparamsout[[m]]=gfit@fit$coef
  
  gfit2 <- ugarchfit(sim.spec2, err, solver="solnp")
  fits2[[m]]=gfit2
  resids2[[m]]=as.numeric(residuals(gfit2, standardize="TRUE"))
  garchparamsout2[[m]]=gfit2@fit$coef
  
  gfit3 <- ugarchfit(sim.spec3, err, solver="solnp")
  fits3[[m]]=gfit3
  resids3[[m]]=as.numeric(residuals(gfit3, standardize="TRUE"))
  garchparamsout3[[m]]=gfit3@fit$coef
}

## Investigate fits1, fits2, fits3 here.

fits[[1]]

LM1=matrix(0,29,3)
LM2=matrix(0,29,3)
LM3=matrix(0,29,3)

wttest = TRUE ## True is LM ARCH Colin Gallagher, FALSE is Li Mak.

for (m in 1:29) {

LM1[m,1]=Weighted.LM.test(resids[[m]], rep(1,length(resids[[m]])), lag = 3,
                 type = c("correlation"),
                 fitdf = 2, weighted = wttest)$p.value

LM1[m,2]=Weighted.LM.test(resids2[[m]], rep(1,length(resids2[[m]])), lag = 4,
                 type = c("correlation"),
                 fitdf = 2, weighted = wttest)$p.value

LM1[m,3]=Weighted.LM.test(resids3[[m]], rep(1,length(resids3[[m]])), lag = 5,
                           type = c("correlation"),
                           fitdf = 2, weighted = wttest)$p.value

LM2[m,1]=Weighted.LM.test(resids2[[m]], rep(1,length(resids2[[m]])), lag = 4,
                          type = c("correlation"),
                          fitdf = 3, weighted = wttest)$p.value

LM2[m,2]=Weighted.LM.test(resids2[[m]], rep(1,length(resids2[[m]])), lag = 5,
                          type = c("correlation"),
                          fitdf = 3, weighted = wttest)$p.value

LM2[m,3]=Weighted.LM.test(resids2[[m]], rep(1,length(resids2[[m]])), lag = 6,
                          type = c("correlation"),
                          fitdf = 3, weighted = wttest)$p.value

LM3[m,1]=Weighted.LM.test(resids3[[m]], rep(1,length(resids3[[m]])), lag = 5,
                          type = c("correlation"),
                          fitdf = 4, weighted = wttest)$p.value

LM3[m,2]=Weighted.LM.test(resids3[[m]], rep(1,length(resids3[[m]])), lag = 6,
                          type = c("correlation"),
                          fitdf = 4, weighted = wttest)$p.value

LM3[m,3]=Weighted.LM.test(resids3[[m]], rep(1,length(resids3[[m]])), lag = 7,
                          type = c("correlation"),
                          fitdf = 4, weighted = wttest)$p.value
}

## Serial autocorrelation in squared std. errors at lags 3, 4
sum(LM1[,1]<0.05) ## Lag 3
sum(LM1[,2]<0.05) ## Lag 4
sum(LM1[,3]<0.05) ## Lag 5

## Again, at lags 4, 5, 6
sum(LM2[,1]<0.05) ## Lag 4
sum(LM2[,2]<0.05) ## Lag 5
sum(LM2[,3]<0.05) ## Lag 6

## Now we've got it taken care of!
sum(LM3[,1]<0.05) ## Lag 5
sum(LM3[,2]<0.05) ## Lag 6
sum(LM3[,3]<0.05) ## Lag 7

## Repeat with wttest = TRUE, then with wttest = FALSE.
## Results are qualitiatively similar.

## Go with EGARCH(3,1).
## Colin and Gallagher, with Li and Mak as well.

fits = fits3
resids = resids3
garchparamsout = garchparamsout3

############################################
## Save workspace again.

save.image(file="DataGARCHThirdRewritePostFit.RData")

########################################

## Check ACF of residuals, and also squared residuals.
dev.off()
par(mfrow=c(3,3), oma=c(1,1,1,1), mar=c(4,2,4,2))
for (n in 1:3) {
  acf(resids[[use[n]]], main="ACF Residuals",ylim=c(-0.02,0.1))
  acf(resids[[use[n]]]^2, main="ACF Residuals^2",ylim=c(-0.02,0.1))
  qqnorm(resids[[use[n]]], main=paste(station[use[n],1]))
  qqline(resids[[use[n]]])  
}
## Save this as diag1.eps

## This is the conditional standard deviation figure in the manuscript.
## condsd.eps
par(mfrow=c(1,5))
n=1
end2=length(sigma(fits[[use[n]]]))
print(end2)
plot(as.numeric(sigma(fits[[use[n]]]))[(end2-1825):end2], xaxt='n',main=paste(station[use[n],1]), xlab="", type="l",ylab="Conditional SD",ylim=c(2,8))
axis(1,c(1,366,731,1096,1461),labels=c("2009","2010","2011","2012","2013"))

n=2
end2=length(sigma(fits[[use[n]]]))
print(end2)
plot(as.numeric(sigma(fits[[use[n]]]))[(end2-1825):end2], xaxt='n',main=paste(station[use[n],1]), xlab="", type="l",ylab="Conditional SD",ylim=c(2,8))
axis(1,c(1,366,731,1096,1461),labels=c("2009","2010","2011","2012","2013"))
n=3
end2=length(sigma(fits[[use[n]]]))
print(end2)
plot(as.numeric(sigma(fits[[use[n]]]))[(end2-1825):end2], xaxt='n',main=paste(station[use[n],1]), xlab="", type="l",ylab="Conditional SD",ylim=c(2,8))
axis(1,c(1,366,731,1096,1461),labels=c("2009","2010","2011","2012","2013"))
n=4
end2=length(sigma(fits[[use[n]]]))
print(end2)
plot(as.numeric(sigma(fits[[use[n]]]))[(end2-1825):end2], xaxt='n',main=paste(station[use[n],1]), xlab="", type="l",ylab="Conditional SD",ylim=c(2,8))
axis(1,c(1,366,731,1096,1461),labels=c("2009","2010","2011","2012","2013"))
n=5
end2=length(sigma(fits[[use[n]]]))
print(end2)
plot(as.numeric(sigma(fits[[use[n]]]))[(end2-1825):end2], xaxt='n',main=paste(station[use[n],1]), xlab="", type="l",ylab="Conditional SD",ylim=c(2,8))
axis(1,c(1,366,731,1096,1461),labels=c("2009","2010","2011","2012","2013"))
## save as condsd.eps


hstart=array()
for (m in 1:29) {
  keeper=dim(sigma(fits[[m]]))[1]
  hstart[m] = sigma(fits[[m]])[keeper]
}

## Estimate the spatial dependence in the residuals, and will use this to simulate
## spatially dependent white noise later.
rmatrix=matrix(NA,end,29)
# rmatrixf=matrix(NA,end,29)
#for (q in 1:5) {
#  m=use[q]
for (m in 1:29) {
  print(m)
  for (i in start[m]:endpoint[m]) {
    rmatrix[i,m]=resids[[m]][i-start[m]+1]
  # rmatrixf[i,m]=(as.numeric(fitted(fits[[m]]))[i - start[m] + 1]) / (as.numeric(sigma(fits[[m]]))[i])
  }
}

Sigma = cor(rmatrix, use="pairwise.complete.obs")
#Sigma = cor(rmatrixf, use="pairwise.complete.obs")

## This is the figure on the estimated correlations from the ARMA-GARCH processes.
## cor.eps

dev.off()
mu=rep(0,29)
hist(Sigma[upper.tri(Sigma, diag=FALSE)], main="Pearson's correlation estimates",
     xlab="Correlation")

sum(Sigma[upper.tri(Sigma, diag=FALSE)]>0.5)

###################
## Make sure you understand what the "residuals" are for an ARMA-EGARCH model
#m=3
#resids[[m]][20]
#as.numeric(sigma(fits[[m]]))[20]
#garchdata[[m]][20]
#garchdata[[m]][20]/as.numeric(sigma(fits[[m]]))[20]

## Doesn't work for 26, due to AR structure
#resids[[m]][26] ## standardized
#resids[[m]][26]*as.numeric(sigma(fits[[m]]))[26] ## matches unstandardized

#garchdata[[m]][26]/as.numeric(sigma(fits[[m]]))[26]

#as.numeric(sigma(fits[[m]]))[26]
#garchdata[[m]][1:26]

#garchdata[[m]][26] - coef(fits[[m]])[1:25]%*%garchdata[[m]][25:1]
#(garchdata[[m]][26] - coef(fits[[m]])[1:25]%*%garchdata[[m]][25:1]) / as.numeric(sigma(fits[[m]]))[26]

save.image(file="DataGARCHPostFitRewrite.RData")
##############################################
## DataGARCHPostFit.RData saved here
##############################################
## This shows some visualization of the data, sort of an exploratory data analysis

#n=1
#end2=length(sigma(fits[[use[n]]]))

#m=1
#Z=ddf2[start[m]:endpoint[m],m]
#len=length(Z)
#lmtry=lm(Z~xmreg[start[m]:endpoint[m],])
#resids[[m]]=lmtry$resid

## This makes the figure showing Atlanta's daily temperature data, and the conditional
## variance of the residuals.  Notice that the residuals are defined using lmtry, 
## so this is simply exploring the structure of the raw residuals.
## The bottom row means we need the sin/cos terms for the conditional variance.

## atl.eps
dev.off()
par(mfrow=c(2,2))
par(oma=c(1.5,1.1,1.2,1.2))
par(mar=c(1.5,4,4,1))

plot(Z[(end2-1825):end2], type="l", xaxt="n",ylab="Temp (deg F)", xlab="", main=paste(city[m]), col="grey", lwd=0.5)
lines(lmtry$fitted.values[(end2-1825):end2], lwd=2)
axis(1,c(1,366,731,1096,1461),labels=c("2009","2010","2011","2012","2013"))

acf(lmtry$resid[(len-1825):len], main="Residuals")

plot(lmtry$resid[(end2-1825):end2]^2, xaxt="n", ylab="Sq. Resids", xlab="", type="l", col="grey")
axis(1,c(1,366,731,1096,1461),labels=c("2009","2010","2011","2012","2013"))

acf(lmtry$resid[(len-1825):len]^2, main="Sq. Residuals")
## This shows the acf of the residuals and squared residuals, again from the lmtry model.
## The left panel means we need ARMA, the right means GARCH.
## acf.eps

dev.off()
par(mfrow=c(1,2))
acf(lmtry$resid[(len-2000):len], main="Residuals")
acf(lmtry$resid[(len-2000):len]^2, main="Sq. Residuals")

len-2000
dev.off()

plot(Z[(len-2000):len], pch=16,cex=0.3, ylab="Avg. Temp (deg F)", xlab="", main=paste(city[m]))
lines(as.numeric(fitted(gfit)[(len-2000):len]), col="blue", type="l")

## OK, this rat's nest is the main simulator.
##
## lensim is the length of simulations, usually one year or 365 days.
## The total number of simulations will be max(outer)*numsim.  I'm doing it this way to keep the
## objects R needs to hold in memory down to a minimum.
##
## For each city, I simulate one year of the process using the fitted parameters for that city. 
## observe that I am using multivariate normal noise ws, which is spatially dependent.

#library(scatterplot3d)
#library(rgl)

#alp=array()
#bet=array()
#omg=array()
#for (m in 1:29) {
#  omg[m]=garchparamsout[[m]][26]
#  alp[m]=garchparamsout[[m]][27]
#  bet[m]=garchparamsout[[m]][28]
#}
#plot(omg,bet, type="n")
#text(omg,bet, labels=station[,1],cex=0.6)
#plot(alp,bet, type="n")
#text(alp,bet, labels=station[,1],cex=0.6)

#plot3d(omg,bet,alp)
#text(omg,bet,alp, labels=station[,1],cex=0.6)

len=length(Z)
setwd("C:/Users/WFU2012/Desktop/Research/NAAJCode/RewriteSims")
for (outer in 41:50) { # again, the endpoint of outer times numsim gives the total number of simulations.
  numsim=10
  lensim=endp-end
  tempsims=array(0,c(lensim,length(use),numsim))
  zsims=array(0,c(lensim,length(use),numsim))
  hsims=array(0,c(lensim,length(use),numsim)) 
  for (q in 1:numsim) {
    print(outer+q/1000)
    
    wspast = mvrnorm(n = len, mu, Sigma) ## I would use this for parameteric bootstrapping, but it's computationally expensive.
    ws = mvrnorm(n = (lensim + 3), mu, Sigma) ## spatial white noise for predictions
    
    Tsim=numeric()
    Zsim=numeric()
    for (mn in 1:length(use)){
      m=use[mn]
      #print(m)
      wstar = sample(resids[[m]], replace=TRUE)
      
      #Z=ddf2[,m]
      #len=length(Z) ## Need to ultimately produce a simulation of the same length
      
      ##################################################################
      ## 1. Simulate a new EGARCH process to then fit it 
      ## to obtain bootstrapped EGARCH parameter.
      m0sim=meanparamsout[[m]][1]
      mstsim=meanparamsout[[m]][2]
      ms1sim=meanparamsout[[m]][3]
      mc1sim=meanparamsout[[m]][4]
      ms2sim=meanparamsout[[m]][5]
      mc2sim=meanparamsout[[m]][6]
      ms3sim=meanparamsout[[m]][7]
      mc3sim=meanparamsout[[m]][8]
      meansim=c(m0sim,mstsim,ms1sim,mc1sim,ms2sim,mc2sim,ms3sim,mc3sim)
      
      vartmp=data.frame(t(garchparamsout[[m]]))
      
      omegasim=vartmp$omega
      alpha1sim=vartmp$alpha1
      alpha2sim=vartmp$alpha2
      alpha3sim=vartmp$alpha3
      beta1sim=vartmp$beta1
      gamma1sim=vartmp$gamma1
      gamma2sim=vartmp$gamma2
      gamma3sim=vartmp$gamma3
      
      vs1sim=vartmp$vxreg1
      vc1sim=vartmp$vxreg2
      vs2sim=vartmp$vxreg3
      vc2sim=vartmp$vxreg4
      vs3sim=vartmp$vxreg5
      vc3sim=vartmp$vxreg6
      vparsim=c(vs1sim, vc1sim, vs2sim, vc2sim, vs3sim, vc3sim)
      
      ar1sim=vartmp$ar1
      ar2sim=vartmp$ar2
      ar3sim=vartmp$ar3
      lenpast=endpoint[m] - start[m] + 1
      
      as = rep(1, lenpast)
      lhs = rep(3, lenpast)
      Zs = rep(0, lenpast)
      Zs[1:3] =first[,m]

      for (j in 4:lenpast) {
        
        lhs[j] = omegasim + gamma1sim * (abs(wstar[(j-1)]) - sqrt(2/pi)) +
          gamma2sim * (abs(wstar[(j-3)]) - sqrt(2/pi)) +
          gamma3sim * (abs(wstar[(j-3)]) - sqrt(2/pi)) +
          alpha1sim * wstar[(j-1)] + 
          alpha2sim * wstar[(j-2)] + 
          alpha3sim * wstar[(j-3)] + 
          beta1sim * lhs[j-1] + (xvreg%*%vparsim)[j]
        as[j] = wstar[j]*sqrt(exp(lhs[j]))
        Zs[j] = ar1sim*(Zs[j-1]) + ## AR1
          ar2sim*(Zs[j-2]) + ## AR2
          ar3sim*(Zs[j-3]) + ## AR3
          as[j]
      }
      Zskeep=Zs #[start[m]:endpoint[m]]
      
      ## Re-fit EGARCH right here.
      sim.spec3 <- ugarchspec(mean.model=list(armaOrder=c(3,0), include.mean=FALSE),
                              variance.model=list(model="eGARCH", garchOrder=c(3,1),
                                                  external.regressors=xvreg[start[m]:endpoint[m],], variance.targeting=FALSE),
                              distribution.model = "norm",
                              start.pars=list(beta1=0.75, omega=.75)) 
      
      fitstar <- ugarchfit(sim.spec3, Zskeep, solver="solnp")

      ## Save parameters from re-fit.
      vartmp=data.frame(t(fitstar@fit$coef))
    
      ##################################################
      ## 2. Re-simulate historical EGARCH run here.  Addd to Fitted Ss.
      omegasim=vartmp$omega
      alpha1sim=vartmp$alpha1
      alpha2sim=vartmp$alpha2
      alpha3sim=vartmp$alpha3
      beta1sim=vartmp$beta1
      gamma1sim=vartmp$gamma1
      gamma2sim=vartmp$gamma2
      gamma3sim=vartmp$gamma3
      
      vs1sim=vartmp$vxreg1
      vc1sim=vartmp$vxreg2
      vs2sim=vartmp$vxreg3
      vc2sim=vartmp$vxreg4
      vs3sim=vartmp$vxreg5
      vc3sim=vartmp$vxreg6
      vparsim=c(vs1sim, vc1sim, vs2sim, vc2sim, vs3sim, vc3sim)
      
      ar1sim=vartmp$ar1
      ar2sim=vartmp$ar2
      ar3sim=vartmp$ar3
      lenpast=endpoint[m] - start[m] + 1
      
      as = rep(1, lenpast)
      lhs = rep(3, lenpast)
      Zs = rep(0, lenpast)
      Zs[1:3] =first[,m]
      
      wstar = sample(resids[[m]], replace=TRUE) ## May as well...
      for (j in (start[m]+3):endpoint[m]) {
        
        lhs[j] = omegasim + gamma1sim * (abs(wstar[(j-1)]) - sqrt(2/pi)) +
          gamma2sim * (abs(wstar[(j-3)]) - sqrt(2/pi)) +
          gamma3sim * (abs(wstar[(j-3)]) - sqrt(2/pi)) +
          alpha1sim * wstar[(j-1)] + 
          alpha2sim * wstar[(j-2)] + 
          alpha3sim * wstar[(j-3)] + 
          beta1sim * lhs[j-1] + (xvreg%*%vparsim)[j]
        as[j] = wstar[j]*sqrt(exp(lhs[j]))
        Zs[j] = ar1sim*(Zs[j-1]) + ## AR1
          ar2sim*(Zs[j-2]) + ## AR2
          ar3sim*(Zs[j-3]) + ## AR3
          as[j]
      }
      Zskeep=Zs[start[m]:endpoint[m]]
      
      ## Add back on to SS here.
      m0sim=meanparamsout[[m]][1]
      mstsim=meanparamsout[[m]][2]
      ms1sim=meanparamsout[[m]][3]
      mc1sim=meanparamsout[[m]][4]
      ms2sim=meanparamsout[[m]][5]
      mc2sim=meanparamsout[[m]][6]
      ms3sim=meanparamsout[[m]][7]
      mc3sim=meanparamsout[[m]][8]
      meansim=c(m0sim,mstsim,ms1sim,mc1sim,ms2sim,mc2sim,ms3sim,mc3sim)
      Ss = rep(0, lenpast)
      Ss = cbind(1,xmreg[start[m]:endpoint[m],])%*%meansim
      
      newrun = Ss + Zskeep
      
      ##################################################################
      ## 3. Re-fit both mean and EGARCH process to obtain new parameters.
        lmnew=lm(newrun~xmreg[start[m]:endpoint[m],])
        err=lmnew$resid
        ln=length(err)
        last3new=err[(ln-2):ln]
        first3new=err[1:3]

        m0sim=lmnew$coeff[1]
        mstsim=lmnew$coeff[2]
        ms1sim=lmnew$coeff[3]
        mc1sim=lmnew$coeff[4]
        ms2sim=lmnew$coeff[5]
        mc2sim=lmnew$coeff[6]
        ms3sim=lmnew$coeff[7]
        mc3sim=lmnew$coeff[8]
        meansim2=c(m0sim,mstsim,ms1sim,mc1sim,ms2sim,mc2sim,ms3sim,mc3sim)
        
        ## Re-fit EGARCH right here.
        sim.spec3 <- ugarchspec(mean.model=list(armaOrder=c(3,0), include.mean=FALSE),
                                variance.model=list(model="eGARCH", garchOrder=c(3,1),
                                                    external.regressors=xvreg[start[m]:endpoint[m],], variance.targeting=FALSE),
                                distribution.model = "norm",
                                start.pars=list(beta1=0.75, omega=.75)) 
        
        fitstar2 <- ugarchfit(sim.spec3, err, solver="solnp")
        
        ## Save parameters from re-fit.
        vartmp2=data.frame(t(fitstar2@fit$coef))
      
      ##################################################################
      ## Predict forward.  Use a set of spatial errors here.
      omegasim=vartmp2$omega
      alpha1sim=vartmp2$alpha1
      alpha2sim=vartmp2$alpha2
      alpha3sim=vartmp2$alpha3
      beta1sim=vartmp2$beta1
      gamma1sim=vartmp2$gamma1
      gamma2sim=vartmp2$gamma2
      gamma3sim=vartmp2$gamma3
      
      vs1sim=vartmp2$vxreg1
      vc1sim=vartmp2$vxreg2
      vs2sim=vartmp2$vxreg3
      vc2sim=vartmp2$vxreg4
      vs3sim=vartmp2$vxreg5
      vc3sim=vartmp2$vxreg6
      vparsim=c(vs1sim, vc1sim, vs2sim, vc2sim, vs3sim, vc3sim)
      
      ar1sim=vartmp2$ar1
      ar2sim=vartmp2$ar2
      ar3sim=vartmp2$ar3

      ## Using bootstrapped mean parameter, simulate for the future here.
      as = rep(0, lensim+3)
      lhs = rep(3, lensim+3)
      Ss = rep(0, lensim+3)
      Zs = rep(0, lensim+3)
      Zs[1:3]=last[,m]
      Ss = cbind(1,xmregp)%*%meansim2
      as[3] = resids[[m]][length(resids[[m]])]
      lhs[3] = log(hstart[m]^2) 
      for (j in 4:(lensim+3)) {
        lhs[j] = omegasim + gamma1sim * (abs(ws[(j-1),m]) - sqrt(2/pi)) +
          gamma2sim * (abs(ws[(j-2),m]) - sqrt(2/pi)) +
          gamma3sim * (abs(ws[(j-3),m]) - sqrt(2/pi)) +
          alpha1sim*ws[(j-1),m] + 
          alpha2sim*ws[(j-2),m] + 
          alpha3sim*ws[(j-3),m] + 
          beta1sim * lhs[j-1] + (xvreg%*%vparsim)[j]
        as[j] = ws[j,m]*sqrt(exp(lhs[j]))
        Zs[j] = ar1sim*(Zs[j-1]) + ## AR1
          ar2sim*(Zs[j-2]) + ## AR1
          ar3sim*(Zs[j-3]) + ## AR1
          as[j]
      }
      Tsfinal2 = Zs[4:(lensim+3)] + Ss ## Here's the final simulation of daily temps. 
      Tsim=cbind(Tsim,Tsfinal2)
      Zsim=cbind(Zsim,Zs[4:(lensim+3)])
    }
    #Tsim=Tsim[,-1]
    #ZSim=hSim[,-1]
    zsims[,,q]=Zsim
    tempsims[,,q]=Tsim
  } ## ends q loop; the rest once once per 'outer' run
  
  for(q in 1:numsim) {
    out=round(tempsims[,,q],2)
    out2=round(zsims[,,q],2)
    colnames(out)=c("AT","BO","NY","LV","TU")
    colnames(out2)=c("AT","BO","NY","LV","TU")
    name=paste("tempRewrite",outer,"S",q,".csv",sep="")
    name2=paste("ztempRewrite",outer,"S",q,".csv",sep="")
    write.csv(out,file=name)
    write.csv(out2,file=name2)
  }
}  

## Save observed temperature data for test set
ddfout=ddf[(end+1):(endp),use]
colnames(ddfout) = c("AT","BO","NY","LV","TU")
write.csv(ddfout, file = "obstemp.csv")


###############################################
## Do I even need any of this stuff below here?
## I guess I do, it compares a single prediction
## to the observed for the purposes of finding the
## matrix Sigma.
###############################################

## Make predictions
len2 = 365*10
preds=numeric()
q=1
for (q in 1:5) {
  m=use[q]
  meansim=meanparamsout[[m]]
  Sstrain = as.array(cbind(1,xmreg)%*%meansim)[(21535 - 2):21535]
  Sst = cbind(1,xmregp)%*%meansim
  Ss = c(Sstrain,Sst)
  
  Zstrain = ddf[(21535 - 2):21535,m]
  Zs = c(Zstrain,rep(0,len2))
  
  vartmp=data.frame(t(garchparamsout[[m]]))
  ar1sim=vartmp$ar1
  ar2sim=vartmp$ar2
  ar3sim=vartmp$ar3

  for (j in 4:length(Ss)) {
    Zs[j] = Ss[j] + 
      ar1sim*(Zs[j-1] - Ss[j-1]) + 
      ar2sim*(Zs[j-2] - Ss[j-2]) + 
      ar3sim*(Zs[j-3] - Ss[j-3])
  }
preds=cbind(preds,Zs)  
}
preds=preds[4:length(Zs),]
colnames(preds)=c("AT","BO","NY","LV","TU")
write.csv(preds,file="preds.csv")



# warnings()
tre=array()
tresd=array()
trep=array()
for (m in 1:29) {
  Z=ddf2[start[m]:endpoint[m],m]
  lmtry=lm(Z~xmreg[start[m]:endpoint[m],])
  tre[m]=summary(lmtry)$coef[2,1]
  tresd[m]=summary(lmtry)$coef[2,2]
  trep[m]=((summary(lmtry)$coef[2,4])<0.01)*1
}
round(tre*365,3)


